from .base import Database
from .postgres import PostgresDatabase
from .sqlserver import SQLServerDatabase
from omop_lite.settings import settings


def create_database() -> Database:
    """Factory function to create the appropriate database instance."""
    if settings.dialect == "postgresql":
        return PostgresDatabase()
    elif settings.dialect == "mssql":
        return SQLServerDatabase()
    else:
        raise ValueError(f"Unsupported dialect: {settings.dialect}")


__all__ = ["Database", "PostgresDatabase", "SQLServerDatabase", "create_database"]



from sqlalchemy import create_engine, MetaData, text
from importlib.resources import files
import logging
from .base import Database
from omop_lite.settings import settings
from typing import Union
from pathlib import Path
from importlib.abc import Traversable

logger = logging.getLogger(__name__)


class PostgresDatabase(Database):
    def __init__(self) -> None:
        super().__init__()
        self.db_url = f"postgresql+psycopg2://{settings.db_user}:{settings.db_password}@{settings.db_host}:{settings.db_port}/{settings.db_name}"
        self.engine = create_engine(self.db_url)
        self.metadata = MetaData(schema=settings.schema_name)
        self.metadata.reflect(bind=self.engine)
        self.file_path = files("omop_lite.scripts.pg")

    def create_schema(self, schema_name: str) -> None:
        if not self.engine:
            raise RuntimeError("Database engine not initialized")
        with self.engine.connect() as connection:
            connection.execute(text(f'CREATE SCHEMA "{schema_name}"'))
            logger.info(f"Schema '{schema_name}' created.")
            connection.commit()

    def add_constraints(self) -> None:
        """
        Add primary keys, constraints, and indices.
        
        Override to add full-text search.
        """
        super().add_constraints()
        self._add_full_text_search()

    def _add_full_text_search(self) -> None:
        """Add full-text search capabilities to the concept table."""
        if not self.engine:
            raise RuntimeError("Database engine not initialized")

        if not settings.fts_create:
            logger.info("Full-text search creation disabled")
            return

        logger.info("Adding full-text search on concept table")

        # Add the tsvector column
        fts_sql = files("omop_lite.scripts").joinpath("fts.sql")
        self._execute_sql_file(fts_sql)
        logger.info("Added full-text search column")

        # Create the GIN index
        fts_index_sql = self.file_path.joinpath("fts_index.sql")
        self._execute_sql_file(fts_index_sql)
        logger.info("Created full-text search index")

    def _bulk_load(self, table_name: str, file_path: Union[Path, Traversable]) -> None:
        if not self.engine:
            raise RuntimeError("Database engine not initialized")
        
        delimiter = self._get_delimiter()
        quote = self._get_quote()

        with open(str(file_path), "r") as f:
            connection = self.engine.raw_connection()
            try:
                cursor = connection.cursor()
                try:
                    with open(str(file_path), "r") as f:
                        cursor.copy_expert(
                            f"COPY {settings.schema_name}.{table_name} FROM STDIN WITH (FORMAT csv, DELIMITER E'{delimiter}', NULL '', QUOTE E'{quote}', HEADER, ENCODING 'UTF8')",
                            f,
                        )
                    connection.commit()
                finally:
                    cursor.close()
            finally:
                connection.close()



from omop_lite.settings import settings
from omop_lite.db import create_database
import logging
from importlib.metadata import version


def main() -> None:
    """
    Main function to create the OMOP Lite database.

    This function will create the schema if it doesn't exist,
    create the tables, load the data, and run the update migrations.
    """
    logging.basicConfig(level=settings.log_level)
    logger = logging.getLogger(__name__)
    logger.info(f"Starting OMOP Lite {version('omop-lite')}")
    logger.debug(f"Settings: {settings.model_dump()}")
    db = create_database()

    # Handle schema creation if not using 'public'
    if settings.schema_name != "public":
        if db.schema_exists(settings.schema_name):
            logger.info(f"Schema '{settings.schema_name}' already exists")
            return
        else:
            db.create_schema(settings.schema_name)

    # Continue with table creation, data loading, etc.
    db.create_tables()
    db.load_data()
    db.add_constraints()

    logger.info("OMOP Lite database created successfully")


if __name__ == "__main__":
    main()



from sqlalchemy import create_engine, MetaData, text
from importlib.resources import files
import logging
from .base import Database
from omop_lite.settings import settings
from typing import Union
from pathlib import Path
from importlib.abc import Traversable

logger = logging.getLogger(__name__)


class SQLServerDatabase(Database):
    def __init__(self) -> None:
        super().__init__()
        self.db_url = f"mssql+pyodbc://{settings.db_user}:{settings.db_password}@{settings.db_host}:{settings.db_port}/{settings.db_name}?driver=ODBC+Driver+18+for+SQL+Server&TrustServerCertificate=yes"
        self.engine = create_engine(self.db_url)
        self.metadata = MetaData(schema=settings.schema_name)
        self.metadata.reflect(bind=self.engine)
        self.file_path = files("omop_lite.scripts.mssql")

    def create_schema(self, schema_name: str) -> None:
        if not self.engine:
            raise RuntimeError("Database engine not initialized")
        with self.engine.connect() as connection:
            connection.execute(text(f"CREATE SCHEMA [{schema_name}]"))
            logger.info(f"Schema '{schema_name}' created.")
            connection.commit()

    def _bulk_load(self, table_name: str, file_path: Union[Path, Traversable]) -> None:
        if not self.engine:
            raise RuntimeError("Database engine not initialized")

        delimiter = self._get_delimiter()

        with open(str(file_path), "r") as f:
            csv_headers = next(f).strip().split(delimiter)

            connection = self.engine.raw_connection()
            try:
                cursor = connection.cursor()

                columns = ", ".join(f"[{col}]" for col in csv_headers)
                placeholders = ", ".join(["?" for _ in csv_headers])
                insert_sql = f"INSERT INTO {settings.schema_name}.[{table_name}] ({columns}) VALUES ({placeholders})"

                rows = [
                    line.strip().split(delimiter)
                    + [None] * (len(csv_headers) - len(line.strip().split(delimiter)))
                    for line in f
                ]

                cursor.executemany(insert_sql, rows)
                connection.commit()
                cursor.close()
            finally:
                connection.close()
